import React, { useCallback, useState, useMemo } from 'react';
import { scaleTime, extent, scaleLog, max, line, timeFormat, format } from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = { top: 40, left: 80, bottom: 50, right: 80 };

const formatDate = timeFormat('%b %d,%y');
const formatComma = format(',')

const Tooltip = ({ activeRow, className }) => (
  	<text className={className} x={-10} y={-10} text-anchor={'end'}>
        {activeRow.countryName}: {formatComma(activeRow.deathTotal)}{'  '}
      	death{activeRow.deathTotal > 1 ? 's' : ''} as of {formatDate(activeRow.date)}
    </text>
);

export const LineChart = ({ data, height, width }) => {
  const [
    activeRow,
    setActiveRow,
  ] = useState();

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = useMemo(
    () => 
    	data.reduce((accumulator, countryTimeseries) =>
      	accumulator.concat(countryTimeseries),
    	[]	),
    [data]);  // allData is necessary to get domains of yValue (and possibly for xValue)
  const epsilon = 1;
  const xScale = useMemo(
    () => 
    	scaleTime()
    		.domain(extent(allData, xValue))
    		.range([0, innerWidth]),
    [allData, xValue, innerWidth]);

  const yScale = useMemo(
    () => 
    	scaleLog()
    		.domain([epsilon, max(allData, yValue)])
    		.range([innerHeight, 0])
    		.nice(),
    [allData, yValue, innerHeight]);

  const lineGenerator = useMemo(
    () =>
    	line()
    		.x((d) => xScale(xValue(d)))
    		.y((d) => yScale(epsilon + yValue(d))),
    [xScale, yScale, xValue, yValue, epsilon]);

  const mostRecentDate = xScale.domain()[1];

  const handelVoronoiHover = useCallback((d) => {
    setActiveRow(d);
  }, []);
  
  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((countryTimeseries) => {  //note that data is used to get line chart for each country
          return (
            <path
              className="marker-line"
              d={lineGenerator(countryTimeseries)}
            />
          );
        })}
        {activeRow && (
          <>
            <path
              className="marker-line active"
              d={lineGenerator(
                data.find(
                  (countryTimeSeries) =>
                    countryTimeSeries.countryName === activeRow.countryName
                )
              )}
            />
            <g transform={`translate(${lineGenerator.x()(activeRow)}, ${lineGenerator.y()(activeRow)})`}>
            	<circle r={5} />
              <Tooltip activeRow={activeRow} className="tooltip-stroke" />
              <Tooltip activeRow={activeRow} className="tooltip" />       	
            </g>     
          </>
        )}
        <text className="title">
          Global Coronavirus Deaths Over Time By Country
        </text>
        <text
          className="axis-label"
          transform={`translate(-40, ${
            innerHeight / 2
          }) rotate(-90)`}
          text-anchor="middle"
        >
          Cumulative Deaths
        </text>
        <text
          className="axis-label"
          transform={`translate(${innerWidth / 2}, ${
            innerHeight + 25
          })`}
          text-anchor="middle"
          alignment-baseline="hanging"
        >
          Time
        </text>
        <VoronoiOverlay
          margin={margin}
          onHover={handelVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
        />
      </g>
    </svg>
  );
};
